#!/usr/bin/env ratlperl
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2012.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#

# Script to process output from sidwalk and generate fix-up commands for any
# misprotected elements (those with bad owners/groups).
# This must be run on the VOB server host.
# 
# Usage: ratlperl elem-owner-check.pl [-verbose] vobtag command-output-file

if ($^O =~ "Win32") {
    if ($ENV{'CLEARCASEHOME'}){
	$atria_home = $ENV{'CLEARCASEHOME'};
    } else {
	$atria_home = $ENV{'ATRIAHOME'};
    }
    die("Need CLEARCASEHOME or ATRIAHOME variables set to find tools\n") if (!$atria_home);
    if ($ENV{'TEMP'}) {
	$tmp = $ENV{'TEMP'};
    } elsif ($ENV{'TMP'}) {
	$tmp = $ENV{'TMP'};
    } else {
	die("cannot find temporary directory, please set TEMP or TMP\n");
    }
    $ps = "\\";
    $devnull = "nul:";
} else {
    $atria_home = "/opt/rational/clearcase";
    $ps = '/';
    $tmp = "/tmp";
    $devnull = "/dev/null";
}

$usage = "Usage: elem-owner-check.pl [-verbose] vobtag command-output-file\n";
$verbose = shift;
if ($verbose eq "-h") {
  print $usage;
  exit 0;
} elsif ($verbose ne "-verbose") {
    $vobtag = $verbose;
    undef $verbose;
} else {
    $vobtag = shift;
}
$command_out = shift;

if (!defined $vobtag || !defined $command_out) {
  print STDERR $usage;
  exit 1;
}

# Get the owner and group list from the VOB object
# input: VOB tag
$yes = 'YES';
sub get_owner_group($) {
  $vobtag = $_[0];
  open(DESCRIBE,"\"$atria_home${ps}bin${ps}cleartool\" describe vob:$vobtag |") || die("Can't run cleartool to describe VOB $vobtag: $!\n");;
  while (<DESCRIBE>) {
    chomp;
    s/^\s*//;
    if (/^group /) {
      @line = split;
      if (!defined $prigroup) {
        $prigroup = $line[1];
      }
      $vgroups{$line[1]} = $yes;
    }
    if (/^owner /) {
      @line = split;
      $vob_owner = $line[1];
    }
  }
  close(DESCRIBE);
  @vob_groups = keys(%vgroups);
}

open(CMDOUT, ">$command_out") || die("cannot create command output file $command_out: $!\n");
$sidwalk = "$tmp${ps}sidwalk.out.$$";
if (system "\"$atria_home${ps}etc${ps}utils${ps}vob_sidwalk\" -profile $sidwalk $vobtag $tmp${ps}sidwalk.dump.$$ >$devnull") {
    die("cannot run sidwalk command\n");
}

open(SIDWALK,"<$sidwalk") || die("Can't open sidwalk file $sidwalk: $!\n");

&get_owner_group($vobtag);
if ($verbose) {
  print "VOB owned by: $vob_owner\n";
  print "VOB groups: @vob_groups\n";
}
                    
$unknown = '(Account Unknown|NOBODY)';

$nelem = 0;
select STDOUT; $| = 1;
while (<SIDWALK>) {
  next if (!/^(file|directory) element/);
  $nelem++;
  print "\r$nelem elements" if ($nelem % 100 == 0);

  chomp;
  @line = split /,/;
  # @line now has Type, DBID, Owner-name, Owner-ID, Group-Name, Group-ID, mode
  if ($line[2] =~ $unknown) {
    print "unknown element owner: dbid:$line[1]\n" if $verbose; 
    $badusers{$line[1]}  = $yes;
  }
  if ($line[4] =~ $unknown) {
    print "group unknown: dbid:$line[1]\n" if $verbose;
    $badgroups{$line[1]}  = $yes;
  } elsif ($vgroups{$line[4]} ne $yes) {
    print "group $line[4] not in group list: dbid:$line[1]\n" if $verbose;;
    $badgroups{$line[1]}  = $yes;
  }
}
foreach $i (keys %badusers) {
  if ($badgroups{$i} eq $yes) {
    delete $badgroups{$i};
    delete $badusers{$i};
    $badboth{$i} = $yes;
  }
}
close(SIDWALK);
unlink($sidwalk);
unlink("/tmp${ps}sidwalk.dump.$$");

select STDOUT; $| = 0;
print "\r$nelem elements\n";
@ulist = keys(%badusers);
@glist = keys(%badgroups);
@blist = keys(%badboth);


$cmd = "";
sub accumulate {
  my ($pattern, $command, @rest) = @_;
  if ($cmd eq "") {
    $cmd = "$command $pattern";
  } else {
    $cmd .= " $pattern";
  }
  if (length($cmd) > 250) {
    print CMDOUT "$cmd\n";
    $cmd = "";
  }
}

sub printlist($@) {
  my ($pattern, @l) = @_;
  $cmd = "";
  map { 
    &accumulate("dbid:$_\@$vobtag",$pattern);
  } @l;
  if ($cmd ne "") {
    print CMDOUT "$cmd\n";
  }
}

if ($#ulist >= 0) {
  print $#ulist+1;
  print " owner problems\n";
  print "Fix owner on these element DBIDs: @ulist\n" if $verbose;
  &printlist("cleartool protect -chown $vob_owner", @ulist);
}
if ($#glist >= 0) {
  print $#glist+1;
  print " group problems\n";
  print "Fix groups on these element DBIDs: @glist\n" if $verbose;
  &printlist("cleartool protect -chgrp $prigroup", @glist);
}
if ($#blist >= 0) {
  print $#blist+1;
  print " dual owner/group problems\n";
  print "Fix both owner and group on these element DBIDs: @blist\n" if $verbose;
  &printlist("cleartool protect -chown $vob_owner -chgrp $prigroup", @blist);
}
close(CMDOUT);
if ($#blist + $#ulist + $#glist > -3) {
  print "File $command_out contains a command script to change the owner and/or group of elements owned by unknown/invalid owners or groups to $vob_owner and $prigroup\n";
} else {
  print "no element owner/group problems found\n";
  unlink($command_out);
}
